/*******************************************************************
*
*  DESCRIPTION: class Motor 
*
*  AUTHOR: Herrero Gustavo
*
*  EMAIL: mailto://gh4o@dc.uba.ar
*
*  DATE: 19/04/2002
*
*******************************************************************/

#ifndef __MOTOR_H
#define __MOTOR_H

/** include files **/
#include "atomic.h"  // class Atomic
#include "atomicstate.h"
#include "message.h"


/** forward declarations **/

class MotorState : public AtomicState {

public:
	//estado del motor
	int    s_motor;
	int    cantidad_pisos;
	
	MotorState(){};
	virtual ~MotorState(){};

	MotorState& operator=(MotorState& thisState); //Assignment
	void copyState(MotorState *);
	int  getSize() const;

};

class Motor: public Atomic
{
public:
	Motor( const string &name = "Motor" );		// Default constructor

	virtual string className() const ;



protected:
	Model &initFunction();
	Model &externalFunction( const ExternalMessage & );
	Model &internalFunction( const InternalMessage & );
	Model &outputFunction( const CollectMessage & );

	ModelState *allocateState() {
		return new MotorState;
	}

private:

	//Puertos de Entrada
	const Port  &subir;
	const Port  &bajar;
	//Puerto de Salida
	Port        &respuesta;


	// Parametros
	VTime       tiempo_frenado;
	VTime       tiempo_avance;


	const VTime &t_frenado() const ;
	const VTime &t_avance() const ;


	//Shortcut to state members
	int  s_motor() const;
	void s_motor( int );
	
	int  cantidad_pisos() const;
	void cantidad_pisos( int );
	
};	// class Motor

// ** inline ** // 
inline
	string Motor::className() const
{
	return "Motor" ;
}

inline
	const VTime &Motor::t_frenado() const
{
	return tiempo_frenado ;
}


inline
	const VTime &Motor::t_avance() const
{
	return tiempo_avance ;
}

/*******************************************************************
* Shortcuts to state paramters
*********************************************************************/
inline 
int Motor::s_motor() const {
	return ((MotorState*)getCurrentState())->s_motor;
}

inline
void Motor::s_motor( int p) {
	((MotorState*)getCurrentState())->s_motor = p;
}


inline 
int Motor::cantidad_pisos() const {
	return ((MotorState*)getCurrentState())->cantidad_pisos;
}

inline
void Motor::cantidad_pisos( int p) {
	((MotorState*)getCurrentState())->cantidad_pisos = p;
}


#endif   //__MOTOR_H 
