/*******************************************************************
*
*  DESCRIPTION: class Puerta 
*
*  AUTHOR: Herrero Gustavo
*
*  EMAIL: mailto://gh4o@dc.uba.ar
*
*  DATE: 19/04/2002
*
*******************************************************************/

/** include files **/
#include "puerta.h"     // base header
#include "message.h"    // class InternalMessage 
#include "parsimu.h"    // class ParallelMainSimulator
#include "strutil.h"    // str2Value( ... )

#include "deftp1.h"
/*******************************************************************
* Function Name: Puerta
* Description: constructor
********************************************************************/
Puerta::Puerta( const string &name )
    : Atomic( name )
    , abrir( addInputPort( "abrir" ) )
    , cerrar( addInputPort( "cerrar" ) )
    , sensor( addInputPort( "sensor" ) )
    , estado( addOutputPort( "estado" ) )
{
	string tiempoA( ParallelMainSimulator::Instance().getParameter( description(), "tiempo_apertura" ) );
	if( tiempoA != "" )
		tiempo_apertura = tiempoA;
	else
		tiempo_apertura = VTime(0,0,6,0);

	string tiempoAC( ParallelMainSimulator::Instance().getParameter( description(), "tiempo_abrircerrar" ) );
	if( tiempoAC != "" )
		tiempo_abrir_cerrar = tiempoAC;
	else
		tiempo_abrir_cerrar = VTime(0,0,3,5);
}


/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &Puerta::initFunction()
{
	estado_puerta = s_cerrada;
	this->passivate();
	return *this ;
}


/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &Puerta::externalFunction( const ExternalMessage &msg )
{

	if( msg.port() == abrir)
	{
        switch(estado_puerta)
        {    
            case s_cerrada:
                //marcar estado_puerta abriendo
        	    estado_puerta = s_abriendo;
        	    // mandar a abrir
        		holdIn(AtomicState::active, tiempo_abrir_cerrar);
        		break;
            case s_cerrando:
                //marcar estado_puerta abriendo
                estado_puerta = s_abriendo;
                // mandar a abrir 
                holdIn( AtomicState::active, msg.time() - lastChange());
                break;
    	    case s_abierta:
                // reiniciar el time out de puerta abierta
                estado_puerta = s_abriendo;
                holdIn( AtomicState::active, VTime::Zero);
                break;
        }
    }
    
	if( msg.port() == cerrar)
	{   //si la puerta se esta abriendo, se ignora el evento de cerrar por SEGURIDAD
	    // si la puerta estaba abierta, 
        switch(estado_puerta) {    
            case s_abierta:
                //marcar estado_puerta cerrando
                estado_puerta = s_cerrando;
                // mandar a cerrar
                holdIn( AtomicState::active, tiempo_abrir_cerrar);
                break;
            case s_cerrada:
                //solo programar aviso de cerrada.
                holdIn( AtomicState::active, VTime::Zero);
//            default:                // continuar con lo que estaba haciendo
//           	//msg.time() - lastChange() = es el tiempo que transcurri
//                //holdIn( AtomicState::active, msg.time() - lastChange());
//                holdIn( AtomicState::active, tiempo_abrir_cerrar - (msg.time() - lastChange()));
        }
    }

	if( msg.port() == sensor)
	{   //el sensor se activ,  
	    // solo reaccionar si la puerta estaba abierta, o cerrando.
        switch(estado_puerta)
        {    
            case s_abierta:
    	        //  reiniciar el timer de abierta
                holdIn( AtomicState::active, tiempo_apertura);
                break;
            case s_cerrando:
                //marcar estado_puerta abriendo
                estado_puerta = s_abriendo;
                // mandar a abrir 
                holdIn( AtomicState::active, msg.time() - lastChange());
                break;
         }
    }

	return *this;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &Puerta::internalFunction( const InternalMessage & )
{

    switch(estado_puerta)
    {    
        case s_cerrando:
            estado_puerta = s_cerrada;
            passivate();
    		break;
        case s_abierta:
            estado_puerta = s_cerrando;
            holdIn(AtomicState::active, tiempo_abrir_cerrar);
    		break;
        case s_abriendo:
            estado_puerta =s_abierta;
            holdIn(AtomicState::active, tiempo_apertura);
    		break;
       	default:
            passivate();
    		break;
    }

	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &Puerta::outputFunction( const CollectMessage &msg )
{
	int output_var;
	
    switch(estado_puerta)
    {    
        case s_cerrando:
        case s_cerrada:
        	output_var = s_cerrada;
            sendOutput( msg.time(), estado, output_var);
            break;
        case s_abriendo:
        	output_var = s_abierta;
            sendOutput( msg.time(), estado, output_var);
            break;
    }

	return *this ;
}
