/*******************************************************************
*
*  DESCRIPTION: class Unidad de Control 
*
*  AUTHOR: Herrero Gustavo
*
*  EMAIL: mailto://gh4o@dc.uba.ar
*
*  DATE: 19/04/2002
*
*******************************************************************/

#ifndef __UCONTROL_H
#define __UCONTROL_H

/** include files **/
#include <map.h>
#include "atomic.h"     	// class Atomic
#include "atomicstate.h"	// class AtomicState
#include "VTime.hh"


/** forward declarations **/

class UControl: public Atomic
{
public:
	UControl( const string &name = "UControl" );				  // Default constructor


	virtual string className() const
	{return "UControl";}

protected:
	Model &initFunction() ;
	Model &externalFunction( const ExternalMessage & );
	Model &internalFunction( const InternalMessage & );
	Model &outputFunction( const CollectMessage & );

	//No special state variables are required for this model
	ModelState* allocateState() {
		return new AtomicState;
	}

private:
	
	//Parametros del modelo
	int         piso_inicial;
	int         cant_pisos;
	//Variables internas
	int         sentido_elev;
	int         estado_puerta;
	int         estado_motor;
	int         piso_actual;
	int         e_proxima_accion;
	 
	typedef map< int, int, less<int> > ListaPisos ;


	ListaPisos  lstSubir,lstBajar;

	//Puertos de Entrada 
	const Port  &boton_piso;   //indica en que piso tiene una llamada de detenerse
	const Port  &boton_puerta; //indica que se preciono abrir o cerrar la puerta.
	const Port  &est_puerta;   // la puerta le manda el estado de cerrada/abierta.
	const Port  &resp_motor;   // la respuesta del motor. -1 desc 1 piso. 1 asc 1 piso, 0 detenido

	//Puerto de Salida
	//status del ascensor
	Port        &nro_piso_actual;
	Port 		&sentido_elevador;
	//msg para el motor para subir o bajar la cabina.
	Port        &subir_cab;
	Port        &bajar_cab;
	//msg para la puerta
	Port        &abrir_puerta;
	Port        &cerrar_puerta;
	
};	// class UControl



#endif   //__UCONTROL_H 
